import 'dart:async';

import 'package:webfaction_api/src/adapter.dart';
import 'package:webfaction_api/src/api/simple/app.dart';

/// See [AppApi]
class App {
  num id;
  String name;
  String type;
  bool autostart;
  num port;
  bool openPort;
  String extraInfo;
  String machine;

  App(this.name, this.type,
      [this.autostart = false, this.extraInfo = '', this.openPort = false]);

  App.fromMap(Map data) {
    id = data['id'];
    name = data['name'];
    type = data['type'];
    autostart = data['autostart'];
    port = data['port'];
    openPort = data['open_port'];
    extraInfo = data['extra_info'];
    machine = data['machine'];
  }
}

/// Create, read, delete, and list operations for Webfaction operations
///
/// See [https://docs.webfaction.com/xmlrpc-api/apiref.html#applications](https://docs.webfaction.com/xmlrpc-api/apiref.html#applications)
class AppObjectApi extends AppApi {
  AppObjectApi({RpcAdapter rpc, String sessionId})
      : super(rpc: rpc, sessionId: sessionId);

  /// Delegates to [create]
  ///
  /// See [App]
  Future<App> createFromInstance(App app) =>
      create(app.name, app.type, app.autostart, app.extraInfo, app.openPort)
          .then((data) async => App.fromMap(data as Map));

  /// Delegates to [delete]
  Future deleteFromInstance(App app) => delete(app.name);
}

class AppType {
  String name;
  String label;
  String description;

  // Webfaction returns a string here
  String autostart;
  String extraInfo;
  bool openPort;

  AppType(this.name, this.label, this.description, this.autostart,
      this.openPort, this.extraInfo);

  AppType.fromMap(data)
      : this(data['name'], data['label'], data['description'],
            data['autostart'], data['open_port'], data['extra_info']);
}
